# DRPG Analytics
A package for tracking user interaction as analytics or metrics.
- **Metrics**: Single record with a count that is incremented each time the metric type is stored.
- **Analytics**: New records created each time the analytic type is stored, with an optional payload.

## Installation
Add our repository to your `composer.json`
```json
{
  "repositories": [
      {
          "type": "composer",
          "url": "https://composer.drp.digital"
      }
  ]
}
```

You can then install the package:
```bash
composer require drp/analytics
```

## Setup
1. Publish the package files:
```bash
php artisan vendor:publish --provider="Drp\Analytics\AnalyticsServiceProvider"
```

2. Migrate or delete the migrations based on your requirements.
```bash
php artisan migrate
```

3. Add the analyticable trait to any models you wish to have the analytics relationship.
> Does not provide a metrics relationship due to metrics being incremental.
```php
class User
{
    use Analyticable;
}
```

4. Create the analytics middleware group in your http/kernel.php
```php
[
    'analytics' => [
        'throttle:120,1', // Allows for up to 2 analytics per second
        \App\Http\Middleware\EncryptCookies::class,
    ],
];
```

5. Add the JS to any layouts that will use js to post trackable events:
```php
{{ Analytics::renderJs(); }} // Defines a global track() JS function...
```

6. Create your trackers in the analytics config file:
> - You should only use the analytic concept if you require the ability to know "who" performed the trackable action.
> The metric concept is much more performant due to it being a simple increment.
> - You may configure the metric concept to keep a track of its actioners via `metric-store-analyticables`.
> - Undefined trackers will use the `undefined-tracker-concept`.
```php
// In config/analytics.php...
[
    'undefined-tracker-concept' => 'analytic',
    
    'metric-store-analyticables' => false,
    
    'trackers' => [
        'logged-in' => 'analytic',
        'clicked-button' => 'metric',
    ],
];
```

## Usage
This package uses a cookie that stores the required user information to create polymorphic relationships. 
This means you only ever need to provide the trackable name and any payload information.

#### PHP persistence
```php
Analytics::store('logged-in');
Analytics::store('logged-in', ['method' => 'sso']); // With payload
```

#### JS persistence
```javascript
track('clicked-button')
track('clicked-button', {location: 'home'}) // With payload
```

#### PHP retrieval
```php
Analytics::get('logged-in');
Analytics::get('logged-in', function (Builder $query): Builder {
    return $query->whereDate('2020-01-01');
});
```

#### Provided Model methods
Analytic:
```php
Analytic::byType('App\User'); // (Scope) Filter to type.

Analytic::eventCount('clicked-button'); // Record count by event.
Analytic::eventCount('clicked-button', 'App\User'); // Record count by event & type.
Analytic::eventCount('logged-in', 'App\User', true); // Distinct record count by event & type.
```

#### Registering new handlers with the Analytics facade
In the below example, when analytics are using the `eloquent` driver, persistence would use the `CustomEloquentAnalyticHandler` class.
> Custom handlers must implement the `HandlesAnalytics` interface.
```php
Analytics::registerHandler('analytic', 'eloquent', CustomEloquentAnalyticHandler::class);
```

## Testing
```bash
composer test
```

## Importing the Package Locally
To help with development, you can require a local package in a local Laravel project.

Add the following "repositories" key below the "scripts" section in `composer.json` file of your 
Laravel app (replace the "url" with the directory where your package lives):

```json
{
    "scripts": { ... },
    "repositories": [
        {
            "type": "path",
            "url": "../../packages/analytics"
        }
    ]
}
```
You can now require your local package in the Laravel application using your chosen namespace of the package. 
Following our example, this would be:

```bash
composer require drp/analytics
```
Important: you will need to perform a `composer update` in your Laravel application whenever you make changes 
to the `composer.json` file of your package or any providers it registers.

## Security
If you discover any security related issues, please email `devops@drpgroup.com` instead of using the issue tracker.

## Credits
- [Matt Morgan](https://git.drp.digital/Matthew.Morgan)
- [Chris Normansell](https://git.drp.digital/Chris.Normansell)
